﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// Object that represents a Cell in a table.
    /// </summary>
    [Serializable]
    public sealed class Cell : SDONSerializeable, TextInterfaceCore<Cell>, TextInterfaceExtended<Cell>, TextInterfaceAlignment<Cell>
    {        
        [DataMember(Name = "Column")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _column = -1;

        [DataMember(Name = "Row")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _row = -1;

        [DataMember(Name = "Label")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _label = null;

        [DataMember(Name = "TextSize")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _textSize = -1.0;

        [DataMember(Name = "TextBold")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textBold = null;

        [DataMember(Name = "TextItalic")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textItalic = null;

        [DataMember(Name = "TextUnderline")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textUnderline = null;

        [DataMember(Name = "TextFont")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textFont = null;

        [DataMember(Name = "TextColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textColor = null;

        [DataMember(Name = "FillColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _fillColor = null;

        [DataMember(Name = "TextAlignH")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textAlignH = null;

        [DataMember(Name = "TextAlignV")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textAlignV = null;

        [DataMember(Name = "Truncate")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _truncate = -1;

        [DataMember(Name = "Note")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _note = null;

        [DataMember(Name = "NoteIcon")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _noteIcon = null;

        [DataMember(Name = "Hyperlink")]
        [IgnoreIfDefaultValue(Default = null)]
        private Hyperlink _hyperlink = null;

        [DataMember(Name = "TextHyperlink")]
        [IgnoreIfDefaultValue(Default = null)]
        private Hyperlink _textHyperlink = null;

        [DataMember(Name = "Image")]
        [IgnoreIfDefaultValue(Default = null)]
        private Image _image = null;

        [DataMember(Name = "Shape")]
        [IgnoreIfDefaultValue(Default = null)]
        private Shape _shape = null;

        /// <summary>
        /// Required. The column of the cell. Note that the first column is column 1 not column 0.
        /// </summary>
        public int Column
        {
            get { return _column; }
            set { _column = value; }
        }

        /// <summary>
        /// Required. The row of the cell. Note that the first row is row 1 not row 0.
        /// </summary>
        public int Row
        {
            get { return _row; }
            set { _row = value; }
        }

        /// <summary>
        /// The text label inside the cell.
        /// </summary>
        /// <param name="label">Label to set</param>
        /// <returns>Cell</returns>
        public Cell SetLabel(string label)
        {
            _label = label;

            return this;
        }

        /// <summary>
        /// The point size of the text label the specified value. If omitted, the text size is the default for the template.
        /// </summary>
        /// <param name="textSize">size of text</param>
        /// <returns>Cell</returns>
        public Cell SetTextSize(double textSize)
        {
            _textSize = textSize;

            return this;
        }

        /// <summary>
        /// Makes the text label be bold with a value of true, not bold with false, otherwise, if omitted, boldness follows the template default.
        /// </summary>
        /// <param name="textBold">Use bold text</param>
        /// <returns>Cell</returns>
        public Cell SetTextBold(bool textBold )
        {
            _textBold = textBold;

            return this;
        }

        /// <summary>
        /// Makes the text label be italic with a value of true, not italic with false, otherwise, if omitted, italic follows the template default.
        /// </summary>
        /// <param name="textItalic">Make text italicized</param>
        /// <returns>Cell</returns>
        public Cell SetTextItalic(bool textItalic)
        {
            _textItalic = textItalic;

            return this;
        }

        /// <summary>
        /// Makes the text label be underlined with a value of true, not underlined with false, otherwise, if omitted, underlined follows the template default.
        /// </summary>
        /// <param name="textUnderline">Make text appear underlined</param>
        /// <returns>Cell</returns>
        public Cell SetTextUnderline(bool textUnderline)
        {
            _textUnderline = textUnderline;

            return this;
        }

        /// <summary>
        /// The font of the text label the specified value. If omitted, the font is the default for the template. Any font can be defined, but will fall back to system default if font is unavailable
        /// </summary>
        /// <param name="textFont">Font name</param>
        /// <returns>Cell</returns>
        public Cell SetTextFont(string textFont)
        {
            _textFont = textFont;

            return this;
        }

        /// <summary>
        /// The color of the text label of the cell (hex RGB value or alias). If omitted, the color is the default for the template.
        /// </summary>
        /// <param name="textColor">Text color</param>
        /// <returns>Cell</returns>
        public Cell SetTextColor(string textColor)
        {
            _textColor = textColor;

            return this;
        }


        /// <summary>
        /// The fill color of the shape (hex RGB value or alias). If omitted, color is the default for the template.
        /// </summary>
        /// <param name="fillColor">Fill color</param>
        /// <returns>Cell</returns>
        public Cell SetFillColor(string fillColor)
        {
            _fillColor = fillColor;

            return this;
        }

        /// <summary>
        /// Aligns the text label to the left, right or centered in the shape. If omitted, the alignment is the default for the template. A value from the HorizontalAlignments enum.
        /// </summary>
        /// <param name="textAlignH">Horizontal text alignment</param>
        /// <returns>Cell</returns>
        public Cell SetTextAlignH(string textAlignH)
        {
            _textAlignH = textAlignH;

            return this;
        }

        /// <summary>
        /// Aligns the text label to the top, bottom or centered in the shape. If omitted, the alignment is the default for the template. A value from the VerticalAlignments enum.
        /// </summary>
        /// <param name="textAlignV">Vertical text alignment</param>
        /// <returns>Cell</returns>
        public Cell SetTextAlignV(string textAlignV)
        {
            _textAlignV = textAlignV;

            return this;
        }

        /// <summary>
        /// The number of characters shown in the cell before the text is truncated.
        /// </summary>
        /// <param name="truncate">Number of characters after which text is truncated</param>
        /// <returns>Cell</returns>
        public Cell SetTextTruncate(int truncate)
        {
            _truncate = truncate;

            return this;
        }

        /// <summary>
        /// A string to appear as a note attached to a cell.
        /// </summary>
        /// <param name="note">Note text to set on cell</param>
        /// <returns>Cell</returns>
        public Cell SetNote(string note)
        {
            _note = note;

            return this;
        }

        /// <summary>
        /// The icon to use for the note on a cell. Defaults to a post-it note icon. Other options are available in the Icons enum.
        /// </summary>
        /// <param name="noteIcon">Note icon to show on cell</param>
        /// <returns>Cell</returns>
        public Cell SetNoteIcon(string noteIcon)
        {
            _noteIcon = noteIcon;

            return this;
        }

        /// <summary>
        /// 
        /// A hyperlink in the cell.
        /// </summary>
        /// <param name="hyperlink">Hyperlink url</param>
        /// <returns>Cell</returns>
        public Cell SetHyperlink(string hyperlink)
        {
            _hyperlink = new Hyperlink();

            _hyperlink.url =  hyperlink;

            return this;
        }

        /// <summary>
        /// Set a hyperlink on text label of cell
        /// </summary>
        /// <param name="textHyperlink">Hyperlink url</param>
        /// <returns>Cell</returns>
        public Cell SetTextHyperlink(string textHyperlink)
        {
            _textHyperlink = new Hyperlink();

            _textHyperlink.url = textHyperlink;

            return this;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="imageUrl">Url to image</param>
        /// <returns>Cell</returns>
        public Cell SetImage(string imageUrl)
        {
            _image = new Image();

            _image.url = imageUrl;

            return this;
        }

        /// <summary>
        ///  Add a shape to the cell
        /// </summary>
        /// <returns>Cell</returns>
        public Shape AddShape( )
        {
            if (_shape != null)
                return _shape;

            _shape = new Shape();

            return _shape;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public Cell()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal Cell(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
