﻿using SDON.Serialization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace SDON
{
    /// <summary>
    /// Object for containing a table of custom data for a shape, defines the schema of a table and gives it an ID so it can be used in shapes contained in the diagram.
    /// </summary>
    [Serializable]
    public sealed class DataTable : SDONSerializeable
    {
        [DataMember(Name = "ID")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _id = null;

        [DataMember(Name = "TableName")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _tableName = null;

        [DataMember(Name = "Columns")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<DataColumn> _columns = null;

        [DataMember(Name = "Rows")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<DataRow> _rows = null;

        /// <summary>
        /// The unique identifier for the data table so it can be referenced in a shape object.
        /// </summary>
        public string ID
        {
            get { return _id; }
            set { _id = value; }
        }

        /// <summary>
        /// The name to give the table.
        /// </summary>
        public string TableName
        {
            get { return _tableName; }
            set { _tableName = value; }
        }

        /// <summary>
        /// Set the data columns
        /// </summary>
        /// <param name="columns">Column objects</param>
        /// <returns>DataTable</returns>
        public DataTable SetColumns(DataColumn[] columns )
        {
            if (_columns == null)
                _columns = new List<DataColumn>();

            foreach(DataColumn column in columns)
            {
                _columns.Add(column);
            }

            return this;
        }


        /// <summary>
        /// Optional. The data to populate rows in the data table with.
        /// </summary>
        /// <returns>DataRow</returns>
        public DataRow AddDataRow()
        {
            if (_rows == null)
                _rows = new List<DataRow>();

            int id = _rows.Count + 1;

            return AddDataRow(id.ToString());
        }


        /// <summary>
        /// Optional. The data to populate rows in the data table with.
        /// <param name="id">ID of row</param>
        /// <returns>DataRow</returns>
        /// </summary>
        public DataRow AddDataRow(string id)
        {
            if (_rows == null)
                _rows = new List<DataRow>();

            DataRow row = new DataRow();

            _rows.Add(row);

            row.RowID = id;

            return row;
        }

        /// <summary>
        /// Get a row by id
        /// <param name="id">ID of row</param>
        /// <returns>DataRow</returns>
        /// </summary>
        public DataRow GetDataRow(string id)
        {
            if (_rows == null)
                return null;

            // If there's already a row with this ID... just return it
            foreach (DataRow row in _rows)
            {
                if (row.RowID == id)
                    return row;
            }

            return _rows.Find(row => row.RowID == id);
        }


        /// <summary>
        /// Default constructor.
        /// </summary>
        public DataTable()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal DataTable(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
