﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// The root object of a SDON document page.
    /// </summary>
    [Serializable]
    public sealed class GanttDocument : DocumentBase
    {


        /// <summary>
        /// Options for Gantt charts
        /// </summary>
        [DataMember(Name = "GanttOptions")]
        [IgnoreIfDefaultValue(Default = null)]
        private GanttOptions _ganttOptions = null;


        /// <summary>
        /// Column definitions for a Gantt chart.
        /// </summary>
        [DataMember(Name = "GanttColumns")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<GanttColumn> _ganttColumns = null;


        /// <summary>
        /// Set options for Gantt chart
        /// </summary>
        /// <param name="allWorkingDays">Make all days working days</param>
        /// <param name="holidays">Country to use for holidays</param>
        /// <returns>GanttDocument</returns>
        public GanttDocument SetOptions(bool allWorkingDays, string holidays)
        {
            if (_ganttOptions == null)
                _ganttOptions = new GanttOptions();

            _ganttOptions.AllWorkingDays = allWorkingDays;
            _ganttOptions.Holidays = holidays;

            return this;
        }

        /// <summary>
        /// Set specific settings for a gantt chart column
        /// </summary>
        /// <param name="name">"official" name of the column ("Task", "Start")</param>
        /// <param name="title">Alias/rename of the named column</param>
        /// <param name="width">Width of the column</param>
        /// <returns>GanttDocument</returns>
        public GanttDocument SetColumnSettings(string name, string title, int width)
        {

            if (_ganttColumns == null)
                _ganttColumns = new List<GanttColumn>();

            GanttColumn ganttColumn = _ganttColumns.Find(columnSettings => columnSettings.Name == name);

            // Gantt column not found.... add it.
            if (ganttColumn == null)
            {
                ganttColumn = new GanttColumn();
                ganttColumn.Name = name;

                _ganttColumns.Add(ganttColumn);
            }

            // Add settings object if not there
            if (ganttColumn.Settings == null)
                ganttColumn.Settings = new GanttColumnSettings();

            if ((title != null) && (title.Length > 0))
                ganttColumn.Settings.Title = title;

            if (width > 0)
                ganttColumn.Settings.Width = width;

            return this;
        }

        /// <summary>
        /// Add a task to the Gantt chart
        /// </summary>
        /// <param name="fields">Array of fields that contain name/value pairs for the gantt chart row</param>
        /// <returns>DataRow</returns>
        public DataRow AddTask(DataField[] fields)
        {
            if (_useDataTable.Rows == null)
                _useDataTable.Rows = new List<DataRow>();

            DataRow dataRow = _useDataTable.AddDataRow();

            dataRow.SetValues(fields);

            return dataRow;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public GanttDocument() : base()
        {
            _template = SDONTemplates.Gantt; 

            _useDataTable = new DataTableInstance();
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal GanttDocument(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        protected override void Serialize(List<MemberInfo> members, SerializationInfo info, ref StreamingContext context, ref List<string> usedPropertyNames)
        {
            base.Serialize(members, info, ref context, ref usedPropertyNames);
        }
    }
}
