﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// An object representing a segmented line that links two shapes
    /// </summary>
    [Serializable]
    public sealed class Return : SDONSerializeable, TextInterfaceCore<Return>, TextInterfaceExtended<Return>, LineInterface<Return>
    {
        [DataMember(Name = "StartID")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _startID = -1;

        [DataMember(Name = "EndID")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _endID = -1;

        [DataMember(Name = "StartDirection")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _startDirection = null;

        [DataMember(Name = "EndDirection")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _endDirection = null;

        [DataMember(Name = "LinePattern")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _linePattern = null;

        [DataMember(Name = "Label")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _label = null;

        [DataMember(Name = "TextBold")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textBold = null;

        [DataMember(Name = "TextItalic")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textItalic = null;

        [DataMember(Name = "TextUnderline")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textUnderline = null;

        [DataMember(Name = "TextSize")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _textSize = -1.0;

        [DataMember(Name = "TextFont")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textFont = null;

        [DataMember(Name = "TextColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textColor = null;

        [DataMember(Name = "Truncate")]
        [IgnoreIfDefaultValue(Default = -2)]
        private int _truncate = -2;

        [DataMember(Name = "TextHyperlink")]
        [IgnoreIfDefaultValue(Default = null)]
        private Hyperlink _textHyperlink = null;

        [DataMember(Name = "StartArrow")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _startArrow = -1;

        [DataMember(Name = "EndArrow")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _endArrow = -1;

        [DataMember(Name = "LineThick")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _lineThick = -1.0;

        [DataMember(Name = "LineColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _lineColor = null;

        [DataMember(Name = "Curved")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _curved = null;

        /// <summary>
        /// Required. The ID of the starting shape. The starting and ending shapes define the direction of the line as far as any arrowhead is concerned, the arrowhead touches the ending shape.
        /// </summary>
        public int StartID
        {
            get { return _startID; }
            set { _startID = value; }
        }

        /// <summary>
        /// Required. The ID of the ending shape. The starting and ending shapes define the direction of the line as far as any arrowhead is concerned, the arrowhead touches the ending shape.
        /// </summary>
        public int EndID
        {
            get { return _endID; }
            set { _endID = value; }
        }


        /// <summary>
        /// The direction the line leaves the starting shape. The default is Down. A value from the Directions enum.
        /// </summary>
        /// <param name="startDirection">Start direction</param>
        /// <returns>Return</returns>
        public Return SetStartDirection(string startDirection)
        {
            _startDirection = startDirection;

            return this;
        }

        /// <summary>
        /// The direction the line goes out the ending shape. The default is Down. A value from the Directions enum.
        /// </summary>
        /// <param name="endDirection">End direction</param>
        /// <returns>Return</returns>
        public Return SetEndDirection(string endDirection)
        {
            _endDirection = endDirection;

            return this;
        }

        /// <summary>
        /// Set the label on the return line
        /// </summary>
        /// <param name="label">Label to set</param>
        /// <returns>Return</returns>
        public Return SetLabel(string label)
        {
            _label = label;

            return this;
        }

        /// <summary>
        /// Bold value for label    
        /// </summary>
        /// <param name="textBold">Bold text in label</param>
        /// <returns>Return</returns>
        public Return SetTextBold(bool textBold)
        {
            _textBold = textBold;

            return this;
        }

        /// <summary>
        /// Italic text on label   
        /// </summary>
        /// <param name="textItalic">Italic text in label</param>
        /// <returns>Return</returns>
        public Return SetTextItalic(bool textItalic)
        {
            _textItalic = textItalic;

            return this;
        }

        /// <summary>
        /// Underlined text in label   
        /// </summary>
        /// <param name="textUnderline">Text underlined in label</param>
        /// <returns>Return</returns>
        public Return SetTextUnderline(bool textUnderline)
        {
            _textUnderline = textUnderline;

            return this;
        }

        /// <summary>
        /// Point size of text.  
        /// </summary>
        /// <param name="textSize">Size of text</param>
        /// <returns>Return</returns>
        public Return SetTextSize(double textSize)
        {
            _textSize = textSize;

            return this;
        }

        /// <summary>
        /// Set the font            
        /// </summary>
        /// <param name="textFont">Font name of text</param>
        /// <returns>Return</returns>
        public Return SetTextFont(string textFont)
        {
            _textFont = textFont;

            return this;
        }

        /// <summary>
        /// Set the text color of the label.   
        /// </summary>
        /// <param name="textColor">Text color</param>
        /// <returns>Return</returns>
        public Return SetTextColor(string textColor)
        {
            _textColor = textColor;

            return this;
        }

        /// <summary>
        /// Defines the number of characters to allow in a shape before the remaining text is truncated. By default nothing is truncated. Defining Truncate to “-1” turns it off if on by default.
        /// </summary>
        /// <param name="truncate">Number of characters before truncation</param>
        /// <returns>Return</returns>
        public Return SetTextTruncate(int truncate)
        {
            _truncate = truncate;

            return this;
        }

        /// <summary>
        /// Set a hyperlink for the text label   
        /// </summary>
        /// <param name="textHyperlink">Hyperlink URL</param>
        /// <returns>Return</returns>
        public Return SetTextHyperlink(string textHyperlink)
        {
            Hyperlink hl = new Hyperlink();

            hl.url = textHyperlink;

            _textHyperlink = hl;

            return this;
        }
        /// <summary>
        /// The arrowhead that will appear on the beginning of the line. A value from the Arrowheads enum. The default arrowhead depends on the template used.
        /// </summary>
        /// <param name="startArrow">Arrowhead used at start of the line</param>
        /// <returns>Return</returns>
        public Return SetStartArrow(int startArrow)
        {
            _startArrow = startArrow;

            return this;
        }

        /// <summary>
        /// The arrowhead that will appear on the end of the line. A value from the Arrowheads enum. The default arrowhead depends on the template used.
        /// </summary>
        /// <param name="endArrow">Arrowhead used at end of the line</param>
        /// <returns>Return</returns>
        public Return SetEndArrow(int endArrow)
        {
            _endArrow = endArrow;

            return this;
        }

        /// <summary>
        /// The pattern of the line connecting the shapes. A value from the LinePatterns enum.
        /// </summary>
        /// <param name="linePattern">Line pattern</param>
        /// <returns>Return</returns>
        public Return SetLinePattern(string linePattern)
        {
            _linePattern = linePattern;

            return this;
        }

        /// <summary>
        /// The thickness of the line in 1/100”. Otherwise the thickness is the default for the template.
        /// </summary>
        /// <param name="lineThick">Line thickness</param>
        /// <returns>Return</returns>    
        public Return SetLineThickness(double lineThick)
        {
            _lineThick = lineThick;

            return this;
        }

        /// <summary>
        /// Make the line color of the connector the specified RGB value or color alias. Otherwise the color is the default for the template.
        /// </summary>        
        /// <param name="lineColor">Line color</param>
        /// <returns>Return</returns>
        public Return SetLineColor(string lineColor)
        {
            _lineColor = lineColor;

            return this;
        }

        /// <summary>
        /// Set a curved return line
        /// </summary>
        /// <param name="curved">Curved line</param>
        /// <returns>Return</returns>
        public Return Curved(bool curved)
        {
            _curved = curved;

            return this;
        }


        /// <summary>
        /// Default constructor.
        /// </summary>
        public Return()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.
        /// <param name="context">The deserialization context.
        internal Return(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
