﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// Object for containing the properties of a row in a table.
    /// </summary>
    [Serializable]
    public sealed class RowProperties : SDONSerializeable, LineInterface<RowProperties>
    {
        [DataMember(Name = "Index")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _index = -1;

        [DataMember(Name = "LineThick")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _lineThick = -1.0;

        [DataMember(Name = "LineColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _lineColor = null;

        [DataMember(Name = "LinePattern")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _linePattern = null;

        [DataMember(Name = "Height")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _height = -1.0;

        [DataMember(Name = "FixedHeight")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _fixedHeight = null;

        /// <summary>
        /// The index of the row.
        /// </summary>
        public int Index
        {
            get { return _index; }
            set { _index = value; }
        }

        /// <summary>
        /// The thickness of the row borders in 1/100”. If omitted, the thickness is the default for the template.
        /// </summary>
        /// <param name="lineThick">Line thickness</param>
        /// <returns>RowProperties</returns>
        public RowProperties SetLineThickness(double lineThick )
        {
            _lineThick = lineThick;

            return this;
        }

        /// <summary>
        /// The line color of the row borders as a hex RGB value or color alias. If omitted, the color is the default for the template.
        /// </summary>
        /// <param name="lineColor">line color</param>
        /// <returns>RowProperties</returns>
        public RowProperties SetLineColor(string lineColor)
        {
            _lineColor = lineColor;

            return this;
        }

        /// <summary>
        /// The pattern of the line for the row border. Must be a value from the LinePatterns enum.
        /// </summary>
        /// <param name="linePattern">Line pattern</param>
        /// <returns>RowProperties</returns>
        public RowProperties SetLinePattern(string linePattern)
        {
            _linePattern = linePattern;

            return this;
        }

        /// <summary>
        /// The minimum height of the row in 1/100”. The text in cells in the row may force the row to be taller than this height.
        /// </summary>
        /// <param name="height">Height</param>
        /// <returns>RowProperties</returns>
        public RowProperties SetHeight(int height)
        {
            _height = height;

            return this;
        }

        /// <summary>
        /// Makes the row not scale as the shape that contains the table grows.
        /// </summary>
        /// <param name="fixedHeight">Fixed height</param>
        /// <returns>RowProperties</returns>
        public RowProperties FixedHeight(bool fixedHeight )
        {
            _fixedHeight = fixedHeight;

            return this;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public RowProperties()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal RowProperties(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
