﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// Object that represents a shape in a SmartDraw diagram.
    /// </summary>
    [Serializable]
    public sealed class Shape : SDONSerializeable, TextInterfaceCore<Shape>, TextInterfaceExtended<Shape>, TextInterfaceAlignment<Shape>, LineInterface<Shape>
    {
        [DataMember(Name = "ID")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _id = -1;

        [DataMember(Name = "ShapeType")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _shapeType = null;

        [DataMember(Name = "Hide")]
        [IgnoreIfDefaultValue(Default = false)]
        private bool _hide = false;

        [DataMember(Name = "Label")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _label = null;

        [DataMember(Name = "TextBold")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textBold = null;

        [DataMember(Name = "TextItalic")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textItalic = null;

        [DataMember(Name = "TextUnderline")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textUnderline = null;

        [DataMember(Name = "TextSize")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _textSize = -1.0;

        [DataMember(Name = "TextFont")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textFont = null;

        [DataMember(Name = "TextColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textColor = null;

        [DataMember(Name = "TextAlignH")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textAlignH = null;

        [DataMember(Name = "TextAlignV")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textAlignV = null;

        [DataMember(Name = "TextGrow")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textGrow = null;

        [DataMember(Name = "TextMargin")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _textMargin = -1;

        [DataMember(Name = "Truncate")]
        [IgnoreIfDefaultValue(Default = -2)]
        private int _truncate = -2;

        [DataMember(Name = "TextHyperlink")]
        [IgnoreIfDefaultValue(Default = null)]
        private Hyperlink _textHyperlink = null;

        [DataMember(Name = "MinWidth")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _minWidth = -1;

        [DataMember(Name = "MinHeight")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _minHeight = -1;

        [DataMember(Name = "FillColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _fillColor = null;

        [DataMember(Name = "LineThick")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _lineThick = -1.0;

        [DataMember(Name = "LineColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _lineColor = null;

        [DataMember(Name = "LineLabel")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _lineLabel = null;

        [DataMember(Name = "LinePattern")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _linePattern = null;

        [DataMember(Name = "Note")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _note = null;

        [DataMember(Name = "NoteIcon")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _noteIcon = null;

        [DataMember(Name = "ShapeConnectorType")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _shapeConnectorType = null;

        [DataMember(Name = "ShapeConnector")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<ShapeConnector> _shapeConnector = null;

        [DataMember(Name = "Hyperlink")]
        [IgnoreIfDefaultValue(Default = null)]
        private Hyperlink _hyperlink = null;

        [DataMember(Name = "Table")]
        [IgnoreIfDefaultValue(Default = null)]
        private Table _table = null;

        [DataMember(Name = "Image")]
        [IgnoreIfDefaultValue(Default = null)]
        private Image _image = null;

        [DataMember(Name = "ShapeContainer")]
        [IgnoreIfDefaultValue(Default = null)]
        private ShapeContainer _shapeContainer = null;

        [DataMember(Name = "Data")]
        [IgnoreIfDefaultValue(Default = null)]
        private DataTableShapeEntry _data = null;


        /// <summary>
        /// The ID of this shape. IDs are used to specify the paths of Return lines. IDs arbitrary but should be unique and must be greater than zero.        
        /// </summary>
        /// <param name="id">Id</param>
        /// <returns>Shape</returns>
        public Shape SetID(int id)
        {
            _id = id;

            return this;
        }

        /// <summary>
        /// Set the shape type   
        /// </summary>
        /// <param name="shapeType">Type of shape (SDON.ShapeTypes enum)</param>
        /// <returns>Shape</returns>
        public Shape SetShapeType(string shapeType)
        {
            _shapeType = shapeType;

            return this;
        }

        /// <summary>
        /// Hide the shape
        /// </summary>
        /// <param name="hide">Hide shape</param>
        /// <returns>Shape</returns>
        public Shape Hide(bool hide)
        {
            _hide = hide;

            return this;
        }

        /// <summary>
        /// Set the label value
        /// </summary>
        /// <param name="label">Label text to set</param>
        /// <returns>Shape</returns>
        public Shape SetLabel(string label)
        {
            _label = label;

            return this;
        }

        /// <summary>
        /// Bold value for label    
        /// </summary>
        /// <param name="textBold">Bold text in label</param>
        /// <returns>Shape</returns>
        public Shape SetTextBold(bool textBold)
        {
           _textBold = textBold;

            return this;
        }

        /// <summary>
        /// Italic text on label   
        /// </summary>
        /// <param name="textItalic">Italic text in label</param>
        /// <returns>Shape</returns>
        public Shape SetTextItalic( bool textItalic)
        {
            _textItalic = textItalic;

            return this;
        }

        /// <summary>
        /// Underlined text in label   
        /// </summary>
        /// <param name="textUnderline">Text underlined in label</param>
        /// <returns>Shape</returns>
        public Shape SetTextUnderline(bool textUnderline)
        {
            _textUnderline = textUnderline;

            return this;
        }

        /// <summary>
        /// Point size of text.  
        /// </summary>
        /// <param name="textSize">Size of text</param>
        /// <returns>Shape</returns>
        public Shape SetTextSize(double textSize)
        {
            _textSize = textSize;

            return this;
        }

        /// <summary>
        /// Set the font            
        /// </summary>
        /// <param name="textFont">Font name for text</param>
        /// <returns>Shape</returns>
        public Shape SetTextFont(string textFont)
        {
            _textFont = textFont;

            return this;
        }

        /// <summary>
        /// Set the space between edge of shape and text    
        /// </summary>
        /// <param name="textMargin">Margin</param>
        /// <returns>Shape</returns>
        public Shape SetTextMargin(int textMargin)
        {
            _textMargin = textMargin;

            return this;
        }


        /// <summary>
        /// Set the text color of the label.   
        /// </summary>
        /// <param name="textColor">Color of the text</param>
        /// <returns>Shape</returns>
        public Shape SetTextColor(string textColor)
        {
            _textColor = textColor;

            return this;
        }

        /// <summary>
        /// Set horizontal text alignement     
        /// </summary>
        /// <param name="textAlignH">Alignment (one of HorizontalAlignments enums)</param>
        /// <returns>Shape</returns>
        public Shape SetTextAlignH(string textAlignH)
        {
            _textAlignH = textAlignH;

            return this;
        }

        /// <summary>
        /// Set vertical text alignement     
        /// </summary>
        /// <param name="textAlignV">Alignment (one of VerticalAlignments enums)</param>
        /// <returns>Shape</returns>
        public Shape SetTextAlignV(string textAlignV)
        {
            _textAlignV = textAlignV;

            return this;
        }

        /// <summary>
        /// Set the manner in which text grows as added
        /// </summary>
        /// <param name="textGrow">Text grow style/type</param>
        /// <returns>Shape</returns>
        public Shape SetTextGrow(string textGrow)
        {
            _textGrow = textGrow;

            return this;
        }

        /// <summary>
        /// Defines the number of characters to allow in a shape before the remaining text is truncated. By default nothing is truncated. Defining Truncate to “-1” turns it off if on by default.
        /// </summary>
        /// <param name="truncate">Number of characters before truncation</param>
        /// <returns>Shape</returns>
        public Shape SetTextTruncate(int truncate)
        {
            _truncate = truncate;

            return this;
        }

        /// <summary>
        /// Set a hyperlink for the text label   
        /// </summary>
        /// <param name="textHyperlink">Hyperlink URL</param>
        /// <returns>Shape</returns>
        public Shape SetTextHyperlink(string textHyperlink)
        {
            Hyperlink hl = new Hyperlink();

            hl.url = textHyperlink;

            _textHyperlink = hl;

            return this;
        }

        /// <summary>
        /// Set fill color of shape    
        /// </summary>
        /// <param name="fillColor">Fill color.   RBG or color alias</param>
        /// <returns>Shape</returns>
        public Shape SetFillColor(string fillColor)
        {
            _fillColor = fillColor;

            return this;
        }

        /// <summary>
        /// Specifies the initial width of a shape in 1/100" before any text is added. Adding more text than will fit into the shape will grow it according to the TextGrow value.
        /// </summary>
        /// <param name="minWidth">Minimum width</param>
        /// <returns>Shape</returns>
        public Shape SetMinWidth(int minWidth)
        {
            _minWidth = minWidth;

            return this;
        }

        /// <summary>
        /// Specifies the initial height of a shape in 1/100" before any text is added. Adding more text than will fit into the shape will grow it according to the TextGrow value.
        /// </summary>
        /// <param name="minHeight">Minimum height</param>
        /// <returns>Shape</returns>
        public Shape SetMinHeight(int minHeight)
        {
            _minHeight = minHeight;

            return this;
        }

        /// <summary>
        /// The border thickness of the shape in 1/100”. If omitted, the thickness is the default for the template.
        /// </summary>
        /// <param name="lineThick">Thickness of the border</param>
        /// <returns>Shape</returns>
        public Shape SetLineThickness(double lineThick)
        {
            _lineThick = lineThick;

            return this;
        }

        /// <summary>
        /// The border color of the shape (hex RGB value or color alias). If omitted, color is the default for the template.
        /// </summary>
        /// <param name="lineColor">Line color. RBG or color alias</param>
        /// <returns>Shape</returns>
        public Shape SetLineColor(string lineColor)
        {
            _lineColor = lineColor;

            return this;
        }

        /// <summary>
        /// Set line pattern                
        /// </summary>
        /// <param name="linePattern">Line pattern... one of the LinePatterns enum</param>
        /// <returns>Shape</returns>
        public Shape SetLinePattern(string linePattern)
        {
            _linePattern = linePattern;

            return this;
        }

        /// <summary>
        /// Set a label on the connector line segment that touches the shape       
        /// </summary>
        /// <param name="lineLabel">Label for line</param>
        /// <returns>Shape</returns>
        public Shape SetLineLabel(string lineLabel)
        {
            _lineLabel = lineLabel;

            return this;
        }



        /// <summary>
        /// A string to appear as a note attached to a shape.
        /// </summary>
        /// <param name="note">Note text</param>
        /// <returns>Shape</returns>
        public Shape SetNote(string note)
        {
            _note = note;

            return this;
        }

        /// <summary>
        /// The icon to use for the note on a shape. Defaults to a post-it note icon. Other options are available in the Icons enum.
        /// </summary>
        /// <param name="noteIcon">Note icon</param>
        /// <returns>Shape</returns>
        public Shape SetNoteIcon(string noteIcon)
        {
            _noteIcon = noteIcon;

            return this;
        }

        /// <summary>
        /// Add a Shape Connector
        /// </summary>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector AddShapeConnector()
        {
            return AddShapeConnector("");
        }

        /// <summary>
        /// Add a Shape Connector
        /// </summary>
        /// <param name="shapeConnectorArrangemntType">
        /// <returns>Shape</returns>Connector
        public ShapeConnector AddShapeConnector(string shapeConnectorArrangemntType )
        {
            if (_shapeConnector == null)
                _shapeConnector = new List<ShapeConnector>();

            ShapeConnector shapeConnector = new ShapeConnector();

            if ((shapeConnectorArrangemntType != null) && (shapeConnectorArrangemntType.Length > 0))
                shapeConnector.SetArrangement(shapeConnectorArrangemntType);

            _shapeConnector.Add(shapeConnector);

            return shapeConnector;
        }

        /// <summary>
        /// Add a Shape Container
        /// </summary>
        /// <returns>Shape</returns>Connector
        public ShapeContainer AddShapeContainer()
        {
            return AddShapeContainer("");
        }

        /// <summary>
        /// Add a Shape Container
        /// </summary>
        /// <param name="shapeContainerArrangement">Shape container arrangement</param>
        /// <returns>Shape</returns>Connector
        public ShapeContainer AddShapeContainer(string shapeContainerArrangement)
        {
            if (_shapeContainer != null)
                return _shapeContainer;

            _shapeContainer = new ShapeContainer();

            if ((shapeContainerArrangement != null) && (shapeContainerArrangement.Length > 0))
                _shapeContainer.SetArrangement(shapeContainerArrangement);

            return _shapeContainer;
        }


        /// <summary>
        /// Set a hyperlink for the shape   
        /// </summary>
        /// <param name="hyperlink">hyperlink URL
        /// <returns>Shape</returns>
        public Shape SetHyperlink(string hyperlink)
        {
            Hyperlink hl = new Hyperlink();

            hl.url = hyperlink;

            _hyperlink = hl;

            return this;
        }

        /// <summary>
        /// Add a table
        /// </summary>
        /// <returns>Table</returns>
        public Table AddTable()
        {
            return AddTable(-1, -1);
        }

        /// <summary>
        /// Add a table
        /// </summary>
        /// <param name="rows">Number of rows to set for the table</param>
        /// <param name="cols">Number of columns to set for the table</param>
        /// <returns>Table</returns>
        public Table AddTable(int rows, int cols )
        {
            if (_table != null)
                return _table;

            _table = new Table();

            if (rows > 0)
                _table.Rows = rows;

            if (cols > 0)
                _table.Columns = cols;

            return _table;
        }

        /// <summary>
        /// An image to put into the shape.
        /// </summary>
        /// <param name="imageUrl">Image URL</param>
        /// <returns>Shape</returns>
        public Shape SetImage(string imageUrl)
        {
            _image = new Image();

            _image.url = imageUrl;

            return this;
        }

        /// <summary>
        /// Add information to associate the shape with a data table
        /// </summary>
        /// <param name="tableID">Table id of data</param>
        /// <param name="rowID">Row id of data</param>
        /// <returns>Shape</returns>
        public Shape SetShapeDataRow(string tableID, string rowID )
        {
            if (_data == null)
                _data = new DataTableShapeEntry();

            _data.TableID = tableID;

            _data.RowID = rowID;

            return this;
        }

        /// <summary>
        /// Add a data shape row 
        /// </summary>
        /// <param name="document">Document object</param>
        /// <param name="tableID">Identifier of table</param>
        /// <returns>DataRow</returns>
        public DataRow AddShapeDataRow(Document document, string tableID)
        {
            // Get the table
            DataTable dataTable = document.GetDataTable(tableID);
            if (dataTable == null)
                return null;

            // Add the row
            DataRow dataRow = dataTable.AddDataRow();

            // Set the table and rowid information into the shape's datatable shape entry
            _data = new DataTableShapeEntry();

            _data.TableID = dataTable.ID;

            _data.RowID = dataRow.RowID;

            return dataRow;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public Shape()
        {
  
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal Shape(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        protected override void Serialize(List<System.Reflection.MemberInfo> members, SerializationInfo info, ref StreamingContext context, ref List<string> usedPropertyNames)
        {
            base.Serialize(members, info, ref context, ref usedPropertyNames);
        }
    }
}
