﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// An object that contains an array of shapes that are connected to it by an automatic connector. Defines an automatic connector.
    /// </summary>
    [Serializable]
    public sealed class ShapeConnector : SDONSerializeable, TextInterfaceCore<ShapeConnector>, LineInterface<ShapeConnector>
    {
        [DataMember(Name = "Collapse")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _collapse = null;

        [DataMember(Name = "Direction")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _direction = null;

        [DataMember(Name = "FillColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _fillColor = null;

        [DataMember(Name = "LineThick")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _lineThick = -1.0;

        [DataMember(Name = "LineColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _lineColor = null;

        [DataMember(Name = "LinePattern")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _linePattern = null;

        [DataMember(Name = "Arrangement")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _arrangement = null;

        [DataMember(Name = "Shapes")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<Shape> _shapes = null;

        [DataMember(Name = "ShapeConnectorType")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _shapeConnectorType = null;

        [DataMember(Name = "StartArrow")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _startArrow = -1;

        [DataMember(Name = "EndArrow")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _endArrow = -1;

        [DataMember(Name = "Label")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _label = null;

        [DataMember(Name = "TextFont")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textFont = null;

        [DataMember(Name = "TextBold")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textBold = null;

        [DataMember(Name = "TextItalic")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textItalic = null;

        [DataMember(Name = "TextUnderline")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textUnderline = null;

        [DataMember(Name = "TextSize")]
        [IgnoreIfDefaultValue(Default = -1.0)]
        private double _textSize = -1.0;

        [DataMember(Name = "TextColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textColor = null;

        [DataMember(Name = "DefaultShape")]
        [IgnoreIfDefaultValue(Default = null)]
        private Shape _defaultShape = null;

        /// <summary>
        /// Whether or not to collapse (hide) the connector. The connector is collapsed initially. This applies only to tree-like diagrams (not flowcharts).
        /// </summary>
        /// <param name="collapse">Collapse the connector</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector Collapse(bool collapse )
        {
            _collapse = collapse;

            return this;
        }

        /// <summary>
        /// The direction of the connector from the parent shape. 
        /// 
        /// For Mind Maps this can be Left or Right for the ShapeLists connected to the root shape. All other uses are ignored. The default is “Right”. Mind Maps ignore any more than two ShapeLists for the root shape and  any more than one for other shapes.
        /// 
        /// For Org Charts (trees) the first ShapeList connected to the root shape can be in any direction and this sets the direction of the tree. The default is “Down”. All other values are ignored. Org charts ignore any more than one ShapeList per shape.
        /// 
        /// For Flow Charts any shape can have multiple ShapeLists in any direction. If two or more ShapeLists attached to a single shape have the same direction, they are shown as a split path. The default direction for a ShapeList is “Right”.
        /// <param name="direction">direction of connector (from Directions enum)</param>
        /// <returns>ShapeConnector</returns>
        /// </summary>
        public ShapeConnector SetDirection(string direction )
        {
            _direction = direction;

            return this;
        }

        /// <summary>
        /// The background color for text labels.
        /// </summary>
        /// <param name="fillColor">Color or color alias</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetFillColor(string fillColor)
        {
            _fillColor = fillColor;

            return this;
        }

        /// <summary>
        /// The thickness of the line in 1/100”. If omitted, the thickness is the default for the template.
        /// </summary>
        /// <param name="lineThick">Thickness of line</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetLineThickness(double lineThick )
        {
            _lineThick = lineThick;

            return this;
        }


        /// <summary>
        /// The line color of the connector as a hex RGB value. If omitted, the color is the default for the template.
        /// </summary>
        /// <param name="lineColor">Color or color alias</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetLineColor(string lineColor)
        {
            _lineColor = lineColor;

            return this;
        }

        /// <summary>
        /// The pattern of the line for the connector. Must be a value from the LinePatterns enum.
        /// </summary>
        /// <param name="linePattern">Line pattern</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetLinePattern(string linePattern)
        {
            _linePattern = linePattern;

            return this;
        }

        /// <summary>
        /// The arrangement pattern of the shapes on the connector. Must be a value from the ShapeConnectorArrangement enum.
        /// </summary>
        /// <param name="arrangement">Arrangement</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetArrangement(string arrangement)
        {
            _arrangement = arrangement;

            return this;
        }

        /// <summary>
        /// Add a new shape to the connector
        /// </summary>
        /// <returns>Shape</returns>
        public Shape AddShape()
        {
            if (_shapes == null)
                _shapes = new List<Shape>();

            Shape shape = new Shape();
            _shapes.Add(shape);

            return shape;
        }

        /// <summary>
        /// Defines the way the shape connector will be formatted. If omitted, the template default is used.
        /// </summary>
        /// <param name="shapeConnectorType">Shape Connector type</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetShapeConnectorType(string shapeConnectorType)
        {
            _shapeConnectorType = shapeConnectorType;

            return this;
        }

        /// <summary>
        /// The arrowhead that will appear on the beginning of the line. Must be a value from the Arrowheads enum.
        /// </summary>
        /// <param name="startArrow">Start arrow value</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetStartArrow(int startArrow)
        {
            _startArrow = startArrow;

            return this;
        }

        /// <summary>
        /// The arrowhead that will appear on the end of the line. Must be a value from the Arrowheads enum.
        /// </summary>
        /// <param name="endArrow">End arrow value</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetEndArrow(int endArrow )
        {
            _endArrow = endArrow;

            return this;
        }


        /// <summary>
        /// Display a text label on the connector
        /// </summary>
        /// <param name="label">Label text</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetLabel(string label)
        {
            _label = label;

            return this;
        }

        /// <summary>
        /// Makes the text label be bold with a value of true, not bold with false, otherwise, if omitted, boldness follows the template default.
        /// </summary>
        /// <param name="textBold">Bold text</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetTextBold(bool textBold )
        {
            _textBold = textBold;

            return this;
        }

        /// <summary>
        /// Makes the text label be italic with a value of true, not italic with false, otherwise, if omitted, italic follows the template default.
        /// </summary>
        /// <param name="textItalic">Italic text</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetTextItalic(bool textItalic )
        {
            _textItalic = textItalic;

            return this;
        }

        /// <summary>
        /// Makes the text label be underlined with a value of true, not underlined with false, otherwise, if omitted, underlined follows the template default.
        /// </summary>
        /// <param name="textUnderline">Underlined text</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetTextUnderline(bool textUnderline)
        {
            _textUnderline = textUnderline;

            return this;
        }

        /// <summary>
        /// The point size of the text label the specified value. If omitted, the text size is the default for the template.
        /// </summary>
        /// <param name="textSize">Point size of text</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetTextSize(double textSize )
        {
            _textSize = textSize;

            return this;
        }

        /// <summary>
        /// The font of the text label the specified value. If omitted, the font is the default for the template. Any font can be defined, but will fall back to system default if font is unavailable.
        /// </summary>
        /// <param name="textFont">Font name</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetTextFont(string textFont)
        {
            _textFont = textFont;

            return this;
        }

        /// <summary>
        /// The color of the text label of the connector (hex RGB value or color alias). If omitted, the color is the default for the template.
        /// </summary>
        /// <param name="textColor">RGB color string or color alias</param>
        /// <returns>ShapeConnector</returns>
        public ShapeConnector SetTextColor(string textColor)
        {
            _textColor = textColor;

            return this;
        }

        /// <summary>
        /// Sets the default properties of any child shapes of the connector.
        /// </summary>
        /// <returns>Shape</returns>
        public Shape AddDefaultShape( )
        {
            if (_defaultShape != null)
                return _defaultShape;

            _defaultShape = new Shape();

            return _defaultShape;
        }


        /// <summary>
        /// Default constructor.
        /// </summary>
        public ShapeConnector()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.
        /// <param name="context">The deserialization context.
        internal ShapeConnector(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
