﻿using SDON.Serialization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace SDON
{
    /// <summary>
    /// An object for holding shapes in a grid-like pattern.
    /// </summary>
    [Serializable]
    public sealed class ShapeContainer : SDONSerializeable
    {
        [DataMember(Name = "Shapes")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<Shape> _shapes = null;

        [DataMember(Name = "Arrangement")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _arrangement = null;

        [DataMember(Name = "Wrap")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _wrap = -1;

        [DataMember(Name = "VerticalSpacing")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _verticalSpacing = -1;

        [DataMember(Name = "HorizontalSpacing")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _horizontalSpacing = -1;

        [DataMember(Name = "ShapesAlignH")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _shapesAlignH = null;

        [DataMember(Name = "ShapesAlignV")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _shapesAlignV = null;

        [DataMember(Name = "DefaultShape")]
        [IgnoreIfDefaultValue(Default = null)]
        private Shape _defaultShape = null;


        /// <summary>
        /// Add a new shape to the connector
        /// </summary>
        /// <returns>Shape</returns>
        public Shape AddShape()
        {
            if (_shapes == null)
                _shapes = new List<Shape>();

            Shape shape = new Shape();
            _shapes.Add(shape);

            return shape;
        }

        /// <summary>
        /// Sets the default properties of any child shapes of the connector.
        /// </summary>
        /// <returns>Shape</returns>
        public Shape AddDefaultShape()
        {
            if (_defaultShape != null)
                return _defaultShape;

            _defaultShape = new Shape();

            return _defaultShape;
        }

        /// <summary>
        /// The arrangement grid pattern. Must be a value from ShapeContainerArrangement.
        /// </summary>
        /// <param name="arrangement">Arrangement of shapes</param>
        /// <returns>ShapeContainer</returns>
        public ShapeContainer SetArrangement(string arrangement)
        {
            _arrangement = arrangement;

            return this;
        }

        /// <summary>
        /// This is the maximum number of rows for a "Row" arrangement before it wraps to a new row, or the maximum number of columns for a Column arrangement before it wraps to a new column.
        /// </summary>
        /// <param name="wrap">Max rows before wrap</param>
        /// <returns>ShapeContainer</returns>
        public ShapeContainer SetWrap(int wrap )
        {
            _wrap = wrap;

            return this;
        }


        /// <summary>
        /// Spacing between rows in 1/100". This is inherited by child ShapeContainers.
        /// </summary>
        /// <param name="verticalSpacing">Vertical spacing between shapes</param>
        /// <returns>ShapeContainer</returns>
        public ShapeContainer SetVerticalSpacing(int verticalSpacing)
        {
            _verticalSpacing = verticalSpacing;

            return this;
        }


        /// <summary>
        /// Spacing between columns in 1/100". This is inherited by child ShapeContainers.
        /// </summary>
        /// <param name="horizontalSpacing">Horizontal spacing between shapes</param>
        /// <returns>ShapeContainer</returns>
        public ShapeContainer SetHorizontalSpacing(int horizontalSpacing )
        {
            _horizontalSpacing = horizontalSpacing;

            return this;
        }

        /// <summary>
        /// Controls the positioning of shapes in a column of shapes. Must be a value from ShapeAlignHorizontal.
        /// </summary>
        /// <param name="shapesAlignH">Horizontal alignment option</param>
        /// <returns>ShapeContainer</returns>
        public ShapeContainer SetShapesAlignH(string shapesAlignH )
        {
            _shapesAlignH = shapesAlignH;

            return this;
        }

        /// <summary>
        /// Controls the positioning of a shape in a row of shapes. Must be a value from ShapeAlignVertical
        /// </summary>
        /// <param name="shapesAlignV">Vertical alignment option</param>
        /// <returns>ShapeContainer</returns>
        public ShapeContainer SetShapesAlignV(string shapesAlignV)
        {
            _shapesAlignV = shapesAlignV;

            return this;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public ShapeContainer()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.
        /// <param name="context">The deserialization context.
        internal ShapeContainer(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

    }
}
