﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// Object that represents a table that has been inserted into a shape.
    /// </summary>
    [Serializable]
    public sealed class Table : SDONSerializeable
    {
        [DataMember(Name = "Rows")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _rows = -1;

        [DataMember(Name = "Columns")]
        [IgnoreIfDefaultValue(Default = -1)]
        private int _columns = -1;

        [DataMember(Name = "ColumnWidth")]
        [IgnoreIfDefaultValue(Default = 100)]
        private int _columnWidth = 100;

        [DataMember(Name = "RowHeight")]
        [IgnoreIfDefaultValue(Default = 50)]
        private int _rowHeight = 50;

        [DataMember(Name = "Cells")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<Cell> _cell = null;

        [DataMember(Name = "AlternateRows")]
        [IgnoreIfDefaultValue(Default = null)]
        private TableAlternateRowsColors _alternateRows = null;

        [DataMember(Name = "Join")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<Join> _join = null;

        [DataMember(Name = "ColumnProperties")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<ColumnProperties> _columnProperties = null;

        [DataMember(Name = "RowProperties")]
        [IgnoreIfDefaultValue(Default = null)]
        private List<RowProperties> _rowProperties = null;

        /// <summary>
        /// The number of rows in the table. This can be omitted for a default of 1 if there are columns defined. If neither the number of  rows or columns is defined, the Table object is ignored.
        /// </summary>
        public int Rows
        {
            get { return _rows; }
            set { _rows = value; }
        }

        /// <summary>
        /// The number of columns in the table. This can be omitted for a default of 1 if there are rows defined. If neither the number of  rows or columns is defined, the Table object is ignored.
        /// </summary>
        public int Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }

        /// <summary>
        /// The width of the column in 1/100th inches.
        /// </summary>
        /// <param name="columnWidth">Width of column</param>
        /// <returns>Table</returns>
        public Table SetColumnWidth(int columnWidth )
        {
            _columnWidth = columnWidth;

            return this;
        }

        /// <summary>
        /// The Height of the Row in 1/100th inches.
        /// </summary>
        /// <param name="rowHeight">Height of rows</param>
        /// <returns>Table</returns>
        public Table SetRowHeight(int rowHeight )
        {
            _rowHeight = rowHeight;

            return this;
        }

        /// <summary>
        /// Get a cell by its row and column values
        /// </summary>
        /// <param name="row">Row of cell</param>
        /// <param name="column">Column of cell</param>
        /// <returns>Cell</returns>
        public Cell GetCell(int row, int column )
        {
            return _cell.Find(cell => ((cell.Row == row) && (cell.Column == column)));
        }

        /// <summary>
        /// Add a cell to the table
        /// </summary>
        /// <param name="row">Row of cell</param>
        /// <param name="column">Column of cell</param>
        /// <returns>Cell</returns>
        public Cell AddCell(int row, int column)
        {
            if (_cell == null)
                _cell = new List<Cell>();

            Cell cell = new Cell();

            cell.Row = row;
            cell.Column = column;

            _cell.Add(cell);

            return cell;
        }

        /// <summary>
        /// Set up an alterating rows color scheme
        /// </summary>
        /// <param name="color1">1st color</param>
        /// <param name="color2">2nd color</param>
        /// <returns>Table</returns>
        public Table SetAlternateRowsColors(string color1, string color2)
        {
            if (_alternateRows == null)
                _alternateRows = new TableAlternateRowsColors();

            _alternateRows.Color1 = color1;
            _alternateRows.Color2 = color2;

            return this;
        }


        /// <summary>
        /// List of elements that define a range of table cells to join into one cell.
        /// </summary>
        /// <param name="row">Row to start joining</param>
        /// <param name="column">Column to start joining</param>
        /// <param name="count">Number of cells down to join together</param>
        /// <param name="down">join down (as opposed to across)</param>
        /// <returns>Table</returns>
        private Table JoinCells(int row, int column, int count, bool down )
        {
            if (_join == null)
                _join = new List<Join>();

            Join join = new Join();

            join.Row = row;
            join.Column = column;
            join.N = count;

            join.Down = down;

            _join.Add(join);

            return this;
        }

        /// <summary>
        /// List of elements that define a range of table cells to join into one cell (horizontally)
        /// </summary>
        /// <param name="row">Row to start joining</param>
        /// <param name="column">Column to start joining</param>
        /// <param name="count">Number of cells across to join together</param>
        /// <returns>Table</returns>
        public Table JoinAcross(int row, int column, int count)
        {
            return JoinCells(row, column, count, false);
        }

        /// <summary>
        /// List of elements that define a range of table cells to join into one cell (vertically)
        /// </summary>
        /// <param name="row">Row to start joining</param>
        /// <param name="column">Column to start joining</param>
        /// <param name="count">Number of cells down to join together</param>
        /// <returns>Table</returns>
        public Table JoinDown(int row, int column, int count)
        {
            return JoinCells(row, column, count, true);
        }

        /// <summary>
        /// Add a ColumnProperties for a specific column:  Special properties of individual columns.
        /// </summary>
        /// <param name="index">index of column to add properties for</param>
        /// <returns>ColumnProperties</returns>
        public ColumnProperties AddColumnProperties(int index)
        {
            if (_columnProperties == null)
                _columnProperties = new List<ColumnProperties>();

            ColumnProperties columnProperties = new ColumnProperties();
            columnProperties.Index = index;

            _columnProperties.Add(columnProperties);

            return columnProperties;
        }

        /// <summary>
        /// Add a RowProperties for a specific column:  Special properties of individual row.
        /// </summary>
        /// <param name="index">index of row to add properties for</param>
        /// <returns>RowProperties</returns>
        public RowProperties AddRowProperties(int index)
        {
            if (_rowProperties == null)
                _rowProperties = new List<RowProperties>();

            RowProperties rowProperties = new RowProperties();
            rowProperties.Index = index;

            _rowProperties.Add(rowProperties);

            return rowProperties;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public Table()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal Table(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
